"""
Update-Board is a lightweight CLI tool for managing and delivering updated versions of applications during testing.
"""

__version__ = "0.0.1"
__status__  = "beta1"
__author__  = "zero <zero.kwok@foxmail.com>"

projectName    = 'upboard'
projectHome    = 'https://github.com/ZeroKwok/upboard.git'
projectVersion = f'{__version__} {__status__}'
projectAuthor  = __author__